searchData={"items":[{"type":"module","title":"Arangox","doc":"[![Build Status](https://travis-ci.com/ArangoDB-Community/arangox.svg?branch=master)](https://travis-ci.com/ArangoDB-Community/arangox)\n\nAn implementation of [`DBConnection`](https://hex.pm/packages/db_connection) for\n[ArangoDB](https://www.arangodb.com).\n\nSupports [VelocyStream](https://www.arangodb.com/2017/08/velocystream-async-binary-protocol/),\n[active failover](https://www.arangodb.com/docs/stable/architecture-deployment-modes-active-failover-architecture.html),\ntransactions and streamed cursors.\n\nTested on:\n\n- **ArangoDB** 3.4 - 3.8\n- **Elixir** 1.6 - 1.12\n- **OTP** 20 - 24\n\n[Documentation](https://hexdocs.pm/arangox/readme.html)","ref":"Arangox.html"},{"type":"module","title":"Examples - Arangox","doc":"```elixir\niex> {:ok, conn} = Arangox.start_link(pool_size: 10)\niex> {:ok, %Arangox.Response{status: 200, body: %{\"code\" => 200, \"error\" => false, \"mode\" => \"default\"}}} = Arangox.get(conn, \"/_admin/server/availability\")\niex> {:error, %Arangox.Error{status: 404}} = Arangox.get(conn, \"/invalid\")\niex> %Arangox.Response{status: 200, body: %{\"code\" => 200, \"error\" => false, \"mode\" => \"default\"}} = Arangox.get!(conn, \"/_admin/server/availability\")\niex> {:ok,\niex>   %Arangox.Request{\niex>     body: \"\",\niex>     headers: %{},\niex>     method: :get,\niex>     path: \"/_admin/server/availability\"\niex>   },\niex>   %Arangox.Response{\niex>     status: 200,\niex>     body: %{\"code\" => 200, \"error\" => false, \"mode\" => \"default\"}\niex>   }\niex> } = Arangox.request(conn, :get, \"/_admin/server/availability\")\niex> Arangox.transaction(conn, fn c ->\niex>   stream =\niex>     Arangox.cursor(\niex>       c,\niex>       \"FOR i IN [1, 2, 3] FILTER i == 1 || i == @num RETURN i\",\niex>       %{num: 2},\niex>       properties: [batchSize: 1]\niex>     )\niex>\niex>   Enum.reduce(stream, [], fn resp, acc ->\niex>     acc ++ resp.body[\"result\"]\niex>   end)\niex> end)\n{:ok, [1, 2]}\n```","ref":"Arangox.html#module-examples"},{"type":"module","title":"Clients - Arangox","doc":"#","ref":"Arangox.html#module-clients"},{"type":"module","title":"Velocy - Arangox","doc":"By default, Arangox communicates with _ArangoDB_ via _VelocyStream_, which requires the `:velocy` library:\n\n```elixir\ndef deps do\n  [\n    ...\n    {:arangox, \"~> 0.4.0\"},\n    {:velocy, \"~> 0.1\"}\n  ]\nend\n```\n\nThe default vst chunk size is `30_720`. To change it, you can include the following in your `config/config.exs`:\n\n```elixir\nconfig :arangox, :vst_maxsize, 12_345\n```\n\n#","ref":"Arangox.html#module-velocy"},{"type":"module","title":"HTTP - Arangox","doc":"Arangox has two HTTP clients, `Arangox.GunClient` and `Arangox.MintClient`, they require a json library:\n\n```elixir\ndef deps do\n  [\n    ...\n    {:arangox, \"~> 0.4.0\"},\n    {:jason, \"~> 1.1\"},\n    {:gun, \"~> 1.3.0\"} # or {:mint, \"~> 0.4.0\"}\n  ]\nend\n```\n\n```elixir\nArangox.start_link(client: Arangox.GunClient) # or Arangox.MintClient\n```\n\n```elixir\niex> {:ok, conn} = Arangox.start_link(client: Arangox.GunClient)\niex> {:ok, %Arangox.Response{status: 200, body: nil}} = Arangox.options(conn, \"/\")\n```\n\n**NOTE:** `:mint` doesn't support unix sockets.\n\n**NOTE:** Since `:gun` is an Erlang library, you _might_ need to add it as an extra application in `mix.exs`:\n\n```elixir\ndef application() do\n  [\n    extra_applications: [:logger, :gun]\n  ]\nend\n```\n\nTo use something else, you'd have to implement the `Arangox.Client` behaviour in a\nmodule somewhere and set that instead.\n\nThe default json library is `Jason`. To use a different library, set the `:json_library` config to the module of your choice, i.e:\n\n```elixir\nconfig :arangox, :json_library, Poison\n```\n\n#","ref":"Arangox.html#module-http"},{"type":"module","title":"Benchmarks - Arangox","doc":"**pool size** 10  \n**parallel processes** 1000  \n**system** virtual machine, 1 cpu (not shared), 2GB RAM\n\n| Name         | Latency   |\n| ------------ | --------- |\n| Velocy: GET  | 179.74 ms |\n| Velocy: POST | 201.23 ms |\n| Mint: GET    | 207.00 ms |\n| Mint: POST   | 216.53 ms |\n| Gun: GET     | 222.61 ms |\n| Gun: POST    | 243.65 ms |\n\n Results generated with [`Benchee`](https://hex.pm/packages/benchee).","ref":"Arangox.html#module-benchmarks"},{"type":"module","title":"Start Options - Arangox","doc":"Arangox assumes defaults for the `:endpoints`, `:username` and `:password` options,\nand [`db_connection`](https://hex.pm/packages/db_connection) assumes a default\n`:pool_size` of `1`, so the following:\n\n```elixir\nArangox.start_link()\n```\n\nIs equivalent to:\n\n```elixir\noptions = [\n  endpoints: \"http://localhost:8529\",\n  pool_size: 1\n]\nArangox.start_link(options)\n```","ref":"Arangox.html#module-start-options"},{"type":"module","title":"Endpoints - Arangox","doc":"Unencrypted endpoints can be specified with either `http://` or\n`tcp://`, whereas encrypted endpoints can be specified with `https://`,\n`ssl://` or `tls://`:\n\n```elixir\n\"tcp://localhost:8529\" == \"http://localhost:8529\"\n\"https://localhost:8529\" == \"ssl://localhost:8529\" == \"tls://localhost:8529\"\n\n\"tcp+unix:///tmp/arangodb.sock\" == \"http+unix:///tmp/arangodb.sock\"\n\"https+unix:///tmp/arangodb.sock\" == \"ssl+unix:///tmp/arangodb.sock\" == \"tls+unix:///tmp/arangodb.sock\"\n\n\"tcp://unix:/tmp/arangodb.sock\" == \"http://unix:/tmp/arangodb.sock\"\n\"https://unix:/tmp/arangodb.sock\" == \"ssl://unix:/tmp/arangodb.sock\" == \"tls://unix:/tmp/arangodb.sock\"\n```\n\nThe `:endpoints` option accepts either a binary, or a list of binaries. In the case of a list,\nArangox will try to establish a connection with the first endpoint it can.\n\nIf a connection is established, the availability of the server will be checked (via the _ArangoDB_ api), and\nif an endpoint is in maintenance mode or is a _Follower_ in an _Active Failover_ setup, the connection\nwill be dropped, or in the case of a list, the endpoint skipped.\n\nWith the `:read_only?` option set to `true`, arangox will try to find a server in\n_readonly_ mode instead and add the _x-arango-allow-dirty-read_ header to every request:\n\n```elixir\niex> endpoints = [\"http://localhost:8003\", \"http://localhost:8004\", \"http://localhost:8005\"]\niex> {:ok, conn} = Arangox.start_link(endpoints: endpoints, read_only?: true)\niex> %Arangox.Response{body: body} = Arangox.get!(conn, \"/_admin/server/mode\")\niex> body[\"mode\"]\n\"readonly\"\niex> {:error, %Arangox.Error{status: 403}} = Arangox.post(conn, \"/_api/database\", %{name: \"newDatabase\"})\n```","ref":"Arangox.html#module-endpoints"},{"type":"module","title":"Authentication - Arangox","doc":"#","ref":"Arangox.html#module-authentication"},{"type":"module","title":"Velocy - Arangox","doc":"ArangoDB's VelocyStream endpoints _do not_ read authorization headers, authentication configuration _must_ be \nprovided as options to `Arangox.start_link/1`. \n\nAs a consequence, if you're using bearer auth, there are a couple of caveats to bear in mind:\n\n* New JWT tokens can only be requested in a seperate connection (i.e. during startup before the primary pool\nis initialized)\n* Refreshed tokens can only be authorized by restarting a connection pool\n\n#","ref":"Arangox.html#module-velocy"},{"type":"module","title":"HTTP - Arangox","doc":"When using an HTTP client, Arangox will generate a _Basic_ or _Bearer_ authorization header if the `:auth` option is set to `{:basic, username, password}` or to `{:bearer, token}` respectively, and append it to every request. If the `:auth` option is not explicitly set, no authorization header will be appended.\n\n```elixir\niex> {:ok, conn} = Arangox.start_link(client: Arangox.GunClient, endpoints: \"http://localhost:8001\")\niex> {:error, %Arangox.Error{status: 401}} = Arangox.get(conn, \"/_admin/server/mode\")\n```\n\nThe header value is obfuscated in transfomed requests returned by arangox, for obvious reasons:\n\n```elixir\niex> {:ok, conn} = Arangox.start_link(client: Arangox.GunClient, auth: {:basic, \"root\", \"\"})\niex> {:ok, request, _response} = Arangox.request(conn, :options, \"/\")\niex> request.headers\n%{\"authorization\" => \"...\"}\n```","ref":"Arangox.html#module-http"},{"type":"module","title":"Databases - Arangox","doc":"#","ref":"Arangox.html#module-databases"},{"type":"module","title":"Velocy - Arangox","doc":"If the `:database` option is set, it can be overridden by prepending the path of a\nrequest with `/_db/:value`. If nothing is set, the request will be sent as-is and\n_ArangoDB_ will assume the `_system` database.\n\n#","ref":"Arangox.html#module-velocy"},{"type":"module","title":"HTTP - Arangox","doc":"When using an HTTP client, arangox will prepend `/_db/:value` to the path of every request\nonly if one isn't already prepended. If a `:database` option is not set, nothing is prepended.\n\n```elixir\niex> {:ok, conn} = Arangox.start_link(client: Arangox.GunClient)\niex> {:ok, request, _response} = Arangox.request(conn, :get, \"/_admin/time\")\niex> request.path\n\"/_admin/time\"\niex> {:ok, conn} = Arangox.start_link(database: \"_system\", client: Arangox.GunClient)\niex> {:ok, request, _response} = Arangox.request(conn, :get, \"/_admin/time\")\niex> request.path\n\"/_db/_system/_admin/time\"\niex> {:ok, request, _response} = Arangox.request(conn, :get, \"/_db/_system/_admin/time\")\niex> request.path\n\"/_db/_system/_admin/time\"\n```","ref":"Arangox.html#module-http"},{"type":"module","title":"Headers - Arangox","doc":"Headers can be given as maps:\n\n```elixir\n%{\"header\" => \"value\"}\n```\n\nOr lists of two binary element tuples:\n\n```elixir\n[{\"header\", \"value\"}]\n```\n\nHeaders given to the start option are merged with every request, but will not override\nany of the headers set by Arangox:\n\n```elixir\niex> {:ok, conn} = Arangox.start_link(headers: %{\"header\" => \"value\"})\niex> {:ok, request, _response} = Arangox.request(conn, :get, \"/_api/version\")\niex> request.headers\n%{\"header\" => \"value\"}\n```\n\nHeaders passed to requests will override any of the headers given to the start option\nor set by Arangox:\n\n```elixir\niex> {:ok, conn} = Arangox.start_link(headers: %{\"header\" => \"value\"})\niex> {:ok, request, _response} = Arangox.request(conn, :get, \"/_api/version\", \"\", %{\"header\" => \"new_value\"})\niex> request.headers\n%{\"header\" => \"new_value\"}\n```","ref":"Arangox.html#module-headers"},{"type":"module","title":"Transport - Arangox","doc":"The `:connect_timeout` start option defaults to `5_000`.\n\nTransport options can be specified via `:tcp_opts` and `:ssl_opts`, for unencrypted and\nencrypted connections respectively. When using `:gun` or `:mint`, these options are passed\ndirectly to the `:transport_opts` connect option.\n\nSee [`:gen_tcp.connect_option()`](http://erlang.org/doc/man/gen_tcp.html#type-connect_option)\nfor more information on `:tcp_opts`,\nor [`:ssl.tls_client_option()`](http://erlang.org/doc/man/ssl.html#type-tls_client_option) for `:ssl_opts`.\n\nThe `:client_opts` option can be used to pass client-specific options to `:gun` or `:mint`.\nThese options are merged with and may override values set by arangox. Some options cannot be\noverridden (i.e. `:mint`'s `:mode` option). If `:transport_opts` is set here it will override\neverything given to `:tcp_opts` or `:ssl_opts`, regardless of whether or not a connection is\nencrypted.\n\nSee the `gun:opts()` type in the [gun docs](https://ninenines.eu/docs/en/gun/1.3/manual/gun/)\nor [`connect/4`](https://hexdocs.pm/mint/Mint.HTTP.html#connect/4) in the mint docs for more\ninformation.","ref":"Arangox.html#module-transport"},{"type":"module","title":"Request Options - Arangox","doc":"Request options are handled by and passed directly to `:db_connection`.\nSee [execute/4](https://hexdocs.pm/db_connection/DBConnection.html#execute/4) in the `:db_connection` docs for supported\noptions.\n\nRequest timeouts default to `15_000`.\n\n```elixir\niex> {:ok, conn} = Arangox.start_link()\niex> %Arangox.Response{status: 200, body: %{\"code\" => 200, \"error\" => false, \"mode\" => \"default\"}} = Arangox.get!(conn, \"/_admin/server/availability\", [], timeout: 15_000)\n```","ref":"Arangox.html#module-request-options"},{"type":"module","title":"Contributing - Arangox","doc":"```\nmix format\nmix do format, credo --strict\ndocker-compose up -d\nmix test\n```","ref":"Arangox.html#module-contributing"},{"type":"module","title":"Roadmap - Arangox","doc":"- `:get_endpoints` and `:port_mappings` options\n- An Ecto adapter\n- More descriptive logs","ref":"Arangox.html#module-roadmap"},{"type":"function","title":"Arangox.abort/2","doc":"Aborts a transaction for the given reason.\n\nDelegates to `DBConnection.rollback/2`.","ref":"Arangox.html#abort/2"},{"type":"function","title":"Example - Arangox.abort/2","doc":"iex> {:ok, conn} = Arangox.start_link()\n    iex> Arangox.transaction(conn, fn c ->\n    iex>   Arangox.abort(c, :reason)\n    iex> end)\n    {:error, :reason}","ref":"Arangox.html#abort/2-example"},{"type":"function","title":"Arangox.child_spec/1","doc":"Returns a supervisor child specification for a DBConnection pool.","ref":"Arangox.html#child_spec/1"},{"type":"function","title":"Arangox.cursor/4","doc":"Creates a cursor and returns a `DBConnection.Stream` struct. Results are fetched\nupon enumeration.\n\nThe cursor is created, results fetched, then deleted from the database upon each\nenumeration (not to be confused with iteration). When a cursor is created, an initial\nresult set is fetched from the database. The initial result is returned with the first\niteration, subsequent iterations are fetched lazily.\n\nCan only be used within a `transaction/3` or `run/3` call.\n\nAccepts any of the options accepted by `DBConnection.stream/4`, as well as any of the\nfollowing:\n\n  * `:database` - Sets what database to run the cursor query on\n  * `:properties` - A list or map of additional body attributes to append to the\n  request body when creating the cursor.\n\nDelegates to `DBConnection.stream/4`.","ref":"Arangox.html#cursor/4"},{"type":"function","title":"Example - Arangox.cursor/4","doc":"iex> {:ok, conn} = Arangox.start_link()\n    iex> Arangox.transaction(conn, fn c ->\n    iex>   stream =\n    iex>     Arangox.cursor(\n    iex>       c,\n    iex>       \"FOR i IN [1, 2, 3] FILTER i == 1 || i == @num RETURN i\",\n    iex>       %{num: 2},\n    iex>       properties: [batchSize: 1]\n    iex>     )\n    iex>\n    iex>   first_batch = Enum.at(stream, 0).body[\"result\"]\n    iex>\n    iex>   exhaust_cursor =\n    iex>     Enum.reduce(stream, [], fn resp, acc ->\n    iex>       acc ++ resp.body[\"result\"]\n    iex>     end)\n    iex>\n    iex>   {first_batch, exhaust_cursor}\n    iex> end)\n    {:ok, {[1], [1, 2]}}","ref":"Arangox.html#cursor/4-example"},{"type":"function","title":"Arangox.delete/4","doc":"Runs a DELETE request against a connection pool.\n\nAccepts any of the options accepted by `DBConnection.execute/4`.","ref":"Arangox.html#delete/4"},{"type":"function","title":"Arangox.delete!/4","doc":"Runs a DELETE request against a connection pool. Raises in the case of an error.\n\nAccepts any of the options accepted by `DBConnection.execute!/4`.","ref":"Arangox.html#delete!/4"},{"type":"function","title":"Arangox.get/4","doc":"Runs a GET request against a connection pool.\n\nAccepts any of the options accepted by `DBConnection.execute/4`.","ref":"Arangox.html#get/4"},{"type":"function","title":"Arangox.get!/4","doc":"Runs a GET request against a connection pool. Raises in the case of an error.\n\nAccepts any of the options accepted by `DBConnection.execute!/4`.","ref":"Arangox.html#get!/4"},{"type":"function","title":"Arangox.head/4","doc":"Runs a HEAD request against a connection pool.\n\nAccepts any of the options accepted by `DBConnection.execute/4`.","ref":"Arangox.html#head/4"},{"type":"function","title":"Arangox.head!/4","doc":"Runs a HEAD request against a connection pool. Raises in the case of an error.\n\nAccepts any of the options accepted by `DBConnection.execute!/4`.","ref":"Arangox.html#head!/4"},{"type":"function","title":"Arangox.json_library/0","doc":"Returns the configured JSON library.\n\nTo change the library, include the following in your `config/config.exs`:\n\n    config :arangox, :json_library, Module\n\nDefaults to `Jason`.","ref":"Arangox.html#json_library/0"},{"type":"function","title":"Arangox.options/4","doc":"Runs a OPTIONS request against a connection pool.\n\nAccepts any of the options accepted by `DBConnection.execute/4`.","ref":"Arangox.html#options/4"},{"type":"function","title":"Arangox.options!/4","doc":"Runs a OPTIONS request against a connection pool. Raises in the case of an error.\n\nAccepts any of the options accepted by `DBConnection.execute!/4`.","ref":"Arangox.html#options!/4"},{"type":"function","title":"Arangox.patch/5","doc":"Runs a PATCH request against a connection pool.\n\nAccepts any of the options accepted by `DBConnection.execute/4`.","ref":"Arangox.html#patch/5"},{"type":"function","title":"Arangox.patch!/5","doc":"Runs a PATCH request against a connection pool. Raises in the case of an error.\n\nAccepts any of the options accepted by `DBConnection.execute!/4`.","ref":"Arangox.html#patch!/5"},{"type":"function","title":"Arangox.post/5","doc":"Runs a POST request against a connection pool.\n\nAccepts any of the options accepted by `DBConnection.execute/4`.","ref":"Arangox.html#post/5"},{"type":"function","title":"Arangox.post!/5","doc":"Runs a POST request against a connection pool. Raises in the case of an error.\n\nAccepts any of the options accepted by `DBConnection.execute!/4`.","ref":"Arangox.html#post!/5"},{"type":"function","title":"Arangox.put/5","doc":"Runs a PUT request against a connection pool.\n\nAccepts any of the options accepted by `DBConnection.execute/4`.","ref":"Arangox.html#put/5"},{"type":"function","title":"Arangox.put!/5","doc":"Runs a PUT request against a connection pool. Raises in the case of an error.\n\nAccepts any of the options accepted by `DBConnection.execute!/4`.","ref":"Arangox.html#put!/5"},{"type":"function","title":"Arangox.request/6","doc":"Runs a request against a connection pool.\n\nAccepts any of the options accepted by `DBConnection.execute/4`.","ref":"Arangox.html#request/6"},{"type":"function","title":"Arangox.request!/6","doc":"Runs a request against a connection pool. Raises in the case of an error.\n\nAccepts any of the options accepted by `DBConnection.execute!/4`.","ref":"Arangox.html#request!/6"},{"type":"function","title":"Arangox.run/3","doc":"Acquires a connection from a pool and runs a series of requests or cursors with it.\nIf the connection disconnects, all future calls using that connection reference will\nfail.\n\nRuns can be nested multiple times if the connection reference is used to start a\nnested run (i.e. calling another function that calls this one). The top level run\nfunction will represent the actual run.\n\nDelegates to `DBConnection.run/3`.","ref":"Arangox.html#run/3"},{"type":"function","title":"Example - Arangox.run/3","doc":"result =\n      Arangox.run(conn, fn c  ->\n        Arangox.request!(c, ...)\n      end)","ref":"Arangox.html#run/3-example"},{"type":"function","title":"Arangox.start_link/1","doc":"Starts a connection pool.","ref":"Arangox.html#start_link/1"},{"type":"function","title":"Options - Arangox.start_link/1","doc":"Accepts any of the options accepted by `DBConnection.start_link/2`, as well as any of the\nfollowing:\n\n  * `:endpoints` - Either a single _ArangoDB_ endpoint binary, or a list of endpoints in\n  order of presedence. Each process in a pool will individually attempt to establish a connection\n  with and check the availablility of each endpoint in the order given until an available endpoint\n  is found. Defaults to `\"http://localhost:8529\"`.\n  * `:database` - Arangox will prepend `/_db/:value` to the path of every request that\n  isn't already prepended. If a value is not given, nothing is prepended (_ArangoDB_ will\n  assume the __system_ database).\n  * `:headers` - A map of headers to merge with every request.\n  * `:disconnect_on_error_codes` - A list of status codes that will trigger a forced disconnect.\n  Only integers within the range `400..599` are affected. Defaults to\n  `[401, 405, 503, 505]`.\n  * `:auth` - Configure whether to resolve authorization.\n  Options are: `{:basic, username, password}`, `{:bearer, token}`.\n  * `:read_only?` - Read-only pools will only connect to _followers_ in an active failover\n  setup and add an _x-arango-allow-dirty-read_ header to every request. Defaults to `false`.\n  * `:connect_timeout` - Sets the timeout for establishing connections with a database.\n  * `:tcp_opts` - Transport options for the tcp socket interface (`:gen_tcp` in the case\n  of gun or mint).\n  * `:ssl_opts` - Transport options for the ssl socket interface (`:ssl` in the case of\n  gun or mint).\n  * `:client` - A module that implements the `Arangox.Client` behaviour. Defaults to\n  `Arangox.VelocyClient`.\n  * `:client_opts` - Options for the client library being used. *WARNING*: If `:transport_opts`\n  is set here it will override the options given to `:tcp_opts` _and_ `:ssl_opts`.\n  * `:failover_callback` - A function to call every time arangox fails to establish a\n  connection. This is only called if a list of endpoints is given, regardless of whether or not\n  it's connecting to an endpoint in an _active failover_ setup. Can be either an anonymous function\n  that takes one argument (which is an `%Arangox.Error{}` struct), or a three-element tuple\n  containing arguments to pass to `apply/3` (in which case an `%Arangox.Error{}` struct is always\n  prepended to the arguments).","ref":"Arangox.html#start_link/1-options"},{"type":"function","title":"Arangox.status/1","doc":"Fetches the current status of a transaction from the database and returns its\ncorresponding `DBconnection` status.\n\nDelegates to `DBConnection.status/1`.","ref":"Arangox.html#status/1"},{"type":"function","title":"Arangox.transaction/3","doc":"Acquires a connection from a pool, begins a transaction in the database and runs a\nseries of requests or cursors with it. If the connection disconnects, all future calls\nusing that connection reference will fail.\n\nTransactions can be nested multiple times if the connection reference is used to start a\nnested transactions (i.e. calling another function that calls this one). The top level\ntransaction function will represent the actual transaction and nested transactions will\nbe interpreted as a `run/3`, erego, any collections declared in nested transactions will\nhave no effect.\n\nAccepts any of the options accepted by `DBConnection.transaction/3`, as well as any of the\nfollowing:\n\n  * `:read` - An array of collection names or a single collection name as a binary.\n  * `:write` - An array of collection names or a single collection name as a binary.\n  * `:exclusive` - An array of collection names or a single collection name as a binary.\n  * `:database` - Sets what database to run the transaction on\n  * `:properties` - A list or map of additional body attributes to append to the request\n  body when beginning a transaction.\n\nDelegates to `DBConnection.transaction/3`.","ref":"Arangox.html#transaction/3"},{"type":"function","title":"Example - Arangox.transaction/3","doc":"Arangox.transaction(conn, fn c ->\n      Arangox.status(c) #=> :transaction\n\n      # do stuff\n    end, [\n      write: \"something\",\n      properties: [waitForSync: true]\n    ])","ref":"Arangox.html#transaction/3-example"},{"type":"type","title":"Arangox.bindvars/0","doc":"","ref":"Arangox.html#t:bindvars/0"},{"type":"type","title":"Arangox.body/0","doc":"","ref":"Arangox.html#t:body/0"},{"type":"type","title":"Arangox.client/0","doc":"","ref":"Arangox.html#t:client/0"},{"type":"type","title":"Arangox.conn/0","doc":"","ref":"Arangox.html#t:conn/0"},{"type":"type","title":"Arangox.endpoint/0","doc":"","ref":"Arangox.html#t:endpoint/0"},{"type":"type","title":"Arangox.headers/0","doc":"","ref":"Arangox.html#t:headers/0"},{"type":"type","title":"Arangox.method/0","doc":"","ref":"Arangox.html#t:method/0"},{"type":"type","title":"Arangox.path/0","doc":"","ref":"Arangox.html#t:path/0"},{"type":"type","title":"Arangox.query/0","doc":"","ref":"Arangox.html#t:query/0"},{"type":"type","title":"Arangox.start_option/0","doc":"","ref":"Arangox.html#t:start_option/0"},{"type":"type","title":"Arangox.transaction_option/0","doc":"","ref":"Arangox.html#t:transaction_option/0"},{"type":"module","title":"Arangox.Auth","doc":"","ref":"Arangox.Auth.html"},{"type":"function","title":"Arangox.Auth.validate/1","doc":"","ref":"Arangox.Auth.html#validate/1"},{"type":"type","title":"Arangox.Auth.password/0","doc":"","ref":"Arangox.Auth.html#t:password/0"},{"type":"type","title":"Arangox.Auth.t/0","doc":"","ref":"Arangox.Auth.html#t:t/0"},{"type":"type","title":"Arangox.Auth.token/0","doc":"","ref":"Arangox.Auth.html#t:token/0"},{"type":"type","title":"Arangox.Auth.username/0","doc":"","ref":"Arangox.Auth.html#t:username/0"},{"type":"behaviour","title":"Arangox.Client","doc":"HTTP client behaviour for `Arangox`. Arangox uses client implementations to\nperform all it's connection and execution operations.\n\nTo use an http library other than `:gun` or `:mint`, implement this behaviour\nin a module and pass that module to the `:client` start option.","ref":"Arangox.Client.html"},{"type":"function","title":"Arangox.Client.alive?/1","doc":"","ref":"Arangox.Client.html#alive?/1"},{"type":"callback","title":"Arangox.Client.alive?/1","doc":"","ref":"Arangox.Client.html#c:alive?/1"},{"type":"function","title":"Arangox.Client.close/1","doc":"","ref":"Arangox.Client.html#close/1"},{"type":"callback","title":"Arangox.Client.close/1","doc":"","ref":"Arangox.Client.html#c:close/1"},{"type":"callback","title":"Arangox.Client.connect/2","doc":"Receives an `Arangox.Endpoint` struct and all the start options from `Arangox.start_link/1`.\n\nThe `socket` returned from this callback gets placed in the `:socket` field\nof an `Arango.Connection` struct (a connection's state) to be used by the\nother callbacks as needed. It can be anything, a tuple, another struct, whatever\nthe client needs.\n\nIt's up to the client to consolidate the `:connect_timeout`, `:transport_opts`\nand `:client_opts` options.","ref":"Arangox.Client.html#c:connect/2"},{"type":"function","title":"Arangox.Client.connect/3","doc":"","ref":"Arangox.Client.html#connect/3"},{"type":"function","title":"Arangox.Client.request/2","doc":"","ref":"Arangox.Client.html#request/2"},{"type":"callback","title":"Arangox.Client.request/2","doc":"Receives a `Arangox.Request` struct and a connection's state (an `Arangox.Connection`\nstruct), and returns an `Arangox.Response` struct or error (or exception struct),\nalong with the new state (which doesn't necessarily need to change).\n\nArangox handles the encoding and decoding of request and response bodies, and merging headers.\n\nIf a connection is lost, this may return `{:error, :noproc, state}` to force a disconnect,\notherwise an attempt to reconnect may not be made until the next request hitting this process\nfails.","ref":"Arangox.Client.html#c:request/2"},{"type":"type","title":"Arangox.Client.exception_or_reason/0","doc":"","ref":"Arangox.Client.html#t:exception_or_reason/0"},{"type":"type","title":"Arangox.Client.socket/0","doc":"","ref":"Arangox.Client.html#t:socket/0"},{"type":"module","title":"Arangox.Connection","doc":"`DBConnection` implementation for `Arangox`.","ref":"Arangox.Connection.html"},{"type":"function","title":"Arangox.Connection.new/5","doc":"","ref":"Arangox.Connection.html#new/5"},{"type":"type","title":"Arangox.Connection.failover?/0","doc":"","ref":"Arangox.Connection.html#t:failover?/0"},{"type":"type","title":"Arangox.Connection.t/0","doc":"","ref":"Arangox.Connection.html#t:t/0"},{"type":"module","title":"Arangox.Endpoint","doc":"Utilities for parsing _ArangoDB_ endpoints.\n\n    iex> Endpoint.new(\"http://localhost:8529\")\n    %Arangox.Endpoint{addr: {:tcp, \"localhost\", 8529}, ssl?: false}\n\n    iex> Endpoint.new(\"https://localhost:8529\")\n    %Arangox.Endpoint{addr: {:tcp, \"localhost\", 8529}, ssl?: true}\n\n    iex> Endpoint.new(\"http://unix:/tmp/arangodb.sock\")\n    %Arangox.Endpoint{addr: {:unix, \"/tmp/arangodb.sock\"}, ssl?: false}","ref":"Arangox.Endpoint.html"},{"type":"function","title":"Arangox.Endpoint.new/1","doc":"Parses an endpoint and returns an `%Arangox.Endpoint{}` struct.","ref":"Arangox.Endpoint.html#new/1"},{"type":"type","title":"Arangox.Endpoint.addr/0","doc":"","ref":"Arangox.Endpoint.html#t:addr/0"},{"type":"type","title":"Arangox.Endpoint.t/0","doc":"","ref":"Arangox.Endpoint.html#t:t/0"},{"type":"module","title":"Arangox.GunClient","doc":"An HTTP client implementation of the [`:gun`](https://ninenines.eu/docs/en/gun/1.3/guide \"documentation\") library. Requires [`:gun`](https://hex.pm/packages/gun \"hex.pm\") to be added\nas a dependency.\n\n[__Hex.pm__](https://hex.pm/packages/gun)\n\n[__Documentation__](https://ninenines.eu/docs/en/gun/1.3/guide)","ref":"Arangox.GunClient.html"},{"type":"module","title":"Arangox.MintClient","doc":"An HTTP client implementation of the [`:mint`](https://hexdocs.pm/mint/Mint.HTTP.html \"documentation\") library. Requires [`:mint`](https://hex.pm/packages/mint \"hex.pm\") to be\nadded as a dependency.\n\n[__Hex.pm__](https://hex.pm/packages/mint)\n\n[__Documentation__](https://hexdocs.pm/mint/Mint.HTTP.html)","ref":"Arangox.MintClient.html"},{"type":"module","title":"Arangox.Request","doc":"","ref":"Arangox.Request.html"},{"type":"type","title":"Arangox.Request.t/0","doc":"","ref":"Arangox.Request.html#t:t/0"},{"type":"module","title":"Arangox.Response","doc":"","ref":"Arangox.Response.html"},{"type":"type","title":"Arangox.Response.t/0","doc":"","ref":"Arangox.Response.html#t:t/0"},{"type":"module","title":"Arangox.VelocyClient","doc":"The default client. Implements the [VelocyStream](https://github.com/arangodb/velocystream) protocol.\n\nURI query parsing functions proudly stolen from Plataformatec and\nlicensed under Apache 2.0.","ref":"Arangox.VelocyClient.html"},{"type":"function","title":"Arangox.VelocyClient.maybe_authenticate/1","doc":"","ref":"Arangox.VelocyClient.html#maybe_authenticate/1"},{"type":"function","title":"Arangox.VelocyClient.vst_maxsize/0","doc":"Returns the configured maximum size (in bytes) for a _VelocyPack_ chunk.\n\nTo change the chunk size, include the following in your `config/config.exs`:\n\n    config :arangox, :vst_maxsize, 12_345\n\nCannot be changed during runtime. Defaults to `30_720`.","ref":"Arangox.VelocyClient.html#vst_maxsize/0"},{"type":"exception","title":"Arangox.Error","doc":"","ref":"Arangox.Error.html"},{"type":"type","title":"Arangox.Error.t/0","doc":"","ref":"Arangox.Error.html#t:t/0"},{"type":"extras","title":"Arangox","doc":"# Arangox\n\n[![Build Status](https://travis-ci.com/ArangoDB-Community/arangox.svg?branch=master)](https://travis-ci.com/ArangoDB-Community/arangox)\n\nAn implementation of [`DBConnection`](https://hex.pm/packages/db_connection) for\n[ArangoDB](https://www.arangodb.com).\n\nSupports [VelocyStream](https://www.arangodb.com/2017/08/velocystream-async-binary-protocol/),\n[active failover](https://www.arangodb.com/docs/stable/architecture-deployment-modes-active-failover-architecture.html),\ntransactions and streamed cursors.\n\nTested on:\n\n- **ArangoDB** 3.4 - 3.8\n- **Elixir** 1.6 - 1.12\n- **OTP** 20 - 24\n\n[Documentation](https://hexdocs.pm/arangox/readme.html)","ref":"readme.html"},{"type":"extras","title":"Examples - Arangox","doc":"```elixir\niex> {:ok, conn} = Arangox.start_link(pool_size: 10)\niex> {:ok, %Arangox.Response{status: 200, body: %{\"code\" => 200, \"error\" => false, \"mode\" => \"default\"}}} = Arangox.get(conn, \"/_admin/server/availability\")\niex> {:error, %Arangox.Error{status: 404}} = Arangox.get(conn, \"/invalid\")\niex> %Arangox.Response{status: 200, body: %{\"code\" => 200, \"error\" => false, \"mode\" => \"default\"}} = Arangox.get!(conn, \"/_admin/server/availability\")\niex> {:ok,\niex>   %Arangox.Request{\niex>     body: \"\",\niex>     headers: %{},\niex>     method: :get,\niex>     path: \"/_admin/server/availability\"\niex>   },\niex>   %Arangox.Response{\niex>     status: 200,\niex>     body: %{\"code\" => 200, \"error\" => false, \"mode\" => \"default\"}\niex>   }\niex> } = Arangox.request(conn, :get, \"/_admin/server/availability\")\niex> Arangox.transaction(conn, fn c ->\niex>   stream =\niex>     Arangox.cursor(\niex>       c,\niex>       \"FOR i IN [1, 2, 3] FILTER i == 1 || i == @num RETURN i\",\niex>       %{num: 2},\niex>       properties: [batchSize: 1]\niex>     )\niex>\niex>   Enum.reduce(stream, [], fn resp, acc ->\niex>     acc ++ resp.body[\"result\"]\niex>   end)\niex> end)\n{:ok, [1, 2]}\n```","ref":"readme.html#examples"},{"type":"extras","title":"Clients - Arangox","doc":"#","ref":"readme.html#clients"},{"type":"extras","title":"Velocy - Arangox","doc":"By default, Arangox communicates with _ArangoDB_ via _VelocyStream_, which requires the `:velocy` library:\n\n```elixir\ndef deps do\n  [\n    ...\n    {:arangox, \"~> 0.4.0\"},\n    {:velocy, \"~> 0.1\"}\n  ]\nend\n```\n\nThe default vst chunk size is `30_720`. To change it, you can include the following in your `config/config.exs`:\n\n```elixir\nconfig :arangox, :vst_maxsize, 12_345\n```\n\n#","ref":"readme.html#velocy"},{"type":"extras","title":"HTTP - Arangox","doc":"Arangox has two HTTP clients, `Arangox.GunClient` and `Arangox.MintClient`, they require a json library:\n\n```elixir\ndef deps do\n  [\n    ...\n    {:arangox, \"~> 0.4.0\"},\n    {:jason, \"~> 1.1\"},\n    {:gun, \"~> 1.3.0\"} # or {:mint, \"~> 0.4.0\"}\n  ]\nend\n```\n\n```elixir\nArangox.start_link(client: Arangox.GunClient) # or Arangox.MintClient\n```\n\n```elixir\niex> {:ok, conn} = Arangox.start_link(client: Arangox.GunClient)\niex> {:ok, %Arangox.Response{status: 200, body: nil}} = Arangox.options(conn, \"/\")\n```\n\n**NOTE:** `:mint` doesn't support unix sockets.\n\n**NOTE:** Since `:gun` is an Erlang library, you _might_ need to add it as an extra application in `mix.exs`:\n\n```elixir\ndef application() do\n  [\n    extra_applications: [:logger, :gun]\n  ]\nend\n```\n\nTo use something else, you'd have to implement the `Arangox.Client` behaviour in a\nmodule somewhere and set that instead.\n\nThe default json library is `Jason`. To use a different library, set the `:json_library` config to the module of your choice, i.e:\n\n```elixir\nconfig :arangox, :json_library, Poison\n```\n\n#","ref":"readme.html#http"},{"type":"extras","title":"Benchmarks - Arangox","doc":"**pool size** 10  \n**parallel processes** 1000  \n**system** virtual machine, 1 cpu (not shared), 2GB RAM\n\n| Name         | Latency   |\n| ------------ | --------- |\n| Velocy: GET  | 179.74 ms |\n| Velocy: POST | 201.23 ms |\n| Mint: GET    | 207.00 ms |\n| Mint: POST   | 216.53 ms |\n| Gun: GET     | 222.61 ms |\n| Gun: POST    | 243.65 ms |\n\n Results generated with [`Benchee`](https://hex.pm/packages/benchee).","ref":"readme.html#benchmarks"},{"type":"extras","title":"Start Options - Arangox","doc":"Arangox assumes defaults for the `:endpoints`, `:username` and `:password` options,\nand [`db_connection`](https://hex.pm/packages/db_connection) assumes a default\n`:pool_size` of `1`, so the following:\n\n```elixir\nArangox.start_link()\n```\n\nIs equivalent to:\n\n```elixir\noptions = [\n  endpoints: \"http://localhost:8529\",\n  pool_size: 1\n]\nArangox.start_link(options)\n```","ref":"readme.html#start-options"},{"type":"extras","title":"Endpoints - Arangox","doc":"Unencrypted endpoints can be specified with either `http://` or\n`tcp://`, whereas encrypted endpoints can be specified with `https://`,\n`ssl://` or `tls://`:\n\n```elixir\n\"tcp://localhost:8529\" == \"http://localhost:8529\"\n\"https://localhost:8529\" == \"ssl://localhost:8529\" == \"tls://localhost:8529\"\n\n\"tcp+unix:///tmp/arangodb.sock\" == \"http+unix:///tmp/arangodb.sock\"\n\"https+unix:///tmp/arangodb.sock\" == \"ssl+unix:///tmp/arangodb.sock\" == \"tls+unix:///tmp/arangodb.sock\"\n\n\"tcp://unix:/tmp/arangodb.sock\" == \"http://unix:/tmp/arangodb.sock\"\n\"https://unix:/tmp/arangodb.sock\" == \"ssl://unix:/tmp/arangodb.sock\" == \"tls://unix:/tmp/arangodb.sock\"\n```\n\nThe `:endpoints` option accepts either a binary, or a list of binaries. In the case of a list,\nArangox will try to establish a connection with the first endpoint it can.\n\nIf a connection is established, the availability of the server will be checked (via the _ArangoDB_ api), and\nif an endpoint is in maintenance mode or is a _Follower_ in an _Active Failover_ setup, the connection\nwill be dropped, or in the case of a list, the endpoint skipped.\n\nWith the `:read_only?` option set to `true`, arangox will try to find a server in\n_readonly_ mode instead and add the _x-arango-allow-dirty-read_ header to every request:\n\n```elixir\niex> endpoints = [\"http://localhost:8003\", \"http://localhost:8004\", \"http://localhost:8005\"]\niex> {:ok, conn} = Arangox.start_link(endpoints: endpoints, read_only?: true)\niex> %Arangox.Response{body: body} = Arangox.get!(conn, \"/_admin/server/mode\")\niex> body[\"mode\"]\n\"readonly\"\niex> {:error, %Arangox.Error{status: 403}} = Arangox.post(conn, \"/_api/database\", %{name: \"newDatabase\"})\n```","ref":"readme.html#endpoints"},{"type":"extras","title":"Authentication - Arangox","doc":"#","ref":"readme.html#authentication"},{"type":"extras","title":"Velocy - Arangox","doc":"ArangoDB's VelocyStream endpoints _do not_ read authorization headers, authentication configuration _must_ be \nprovided as options to `Arangox.start_link/1`. \n\nAs a consequence, if you're using bearer auth, there are a couple of caveats to bear in mind:\n\n* New JWT tokens can only be requested in a seperate connection (i.e. during startup before the primary pool\nis initialized)\n* Refreshed tokens can only be authorized by restarting a connection pool\n\n#","ref":"readme.html#velocy"},{"type":"extras","title":"HTTP - Arangox","doc":"When using an HTTP client, Arangox will generate a _Basic_ or _Bearer_ authorization header if the `:auth` option is set to `{:basic, username, password}` or to `{:bearer, token}` respectively, and append it to every request. If the `:auth` option is not explicitly set, no authorization header will be appended.\n\n```elixir\niex> {:ok, conn} = Arangox.start_link(client: Arangox.GunClient, endpoints: \"http://localhost:8001\")\niex> {:error, %Arangox.Error{status: 401}} = Arangox.get(conn, \"/_admin/server/mode\")\n```\n\nThe header value is obfuscated in transfomed requests returned by arangox, for obvious reasons:\n\n```elixir\niex> {:ok, conn} = Arangox.start_link(client: Arangox.GunClient, auth: {:basic, \"root\", \"\"})\niex> {:ok, request, _response} = Arangox.request(conn, :options, \"/\")\niex> request.headers\n%{\"authorization\" => \"...\"}\n```","ref":"readme.html#http"},{"type":"extras","title":"Databases - Arangox","doc":"#","ref":"readme.html#databases"},{"type":"extras","title":"Velocy - Arangox","doc":"If the `:database` option is set, it can be overridden by prepending the path of a\nrequest with `/_db/:value`. If nothing is set, the request will be sent as-is and\n_ArangoDB_ will assume the `_system` database.\n\n#","ref":"readme.html#velocy"},{"type":"extras","title":"HTTP - Arangox","doc":"When using an HTTP client, arangox will prepend `/_db/:value` to the path of every request\nonly if one isn't already prepended. If a `:database` option is not set, nothing is prepended.\n\n```elixir\niex> {:ok, conn} = Arangox.start_link(client: Arangox.GunClient)\niex> {:ok, request, _response} = Arangox.request(conn, :get, \"/_admin/time\")\niex> request.path\n\"/_admin/time\"\niex> {:ok, conn} = Arangox.start_link(database: \"_system\", client: Arangox.GunClient)\niex> {:ok, request, _response} = Arangox.request(conn, :get, \"/_admin/time\")\niex> request.path\n\"/_db/_system/_admin/time\"\niex> {:ok, request, _response} = Arangox.request(conn, :get, \"/_db/_system/_admin/time\")\niex> request.path\n\"/_db/_system/_admin/time\"\n```","ref":"readme.html#http"},{"type":"extras","title":"Headers - Arangox","doc":"Headers can be given as maps:\n\n```elixir\n%{\"header\" => \"value\"}\n```\n\nOr lists of two binary element tuples:\n\n```elixir\n[{\"header\", \"value\"}]\n```\n\nHeaders given to the start option are merged with every request, but will not override\nany of the headers set by Arangox:\n\n```elixir\niex> {:ok, conn} = Arangox.start_link(headers: %{\"header\" => \"value\"})\niex> {:ok, request, _response} = Arangox.request(conn, :get, \"/_api/version\")\niex> request.headers\n%{\"header\" => \"value\"}\n```\n\nHeaders passed to requests will override any of the headers given to the start option\nor set by Arangox:\n\n```elixir\niex> {:ok, conn} = Arangox.start_link(headers: %{\"header\" => \"value\"})\niex> {:ok, request, _response} = Arangox.request(conn, :get, \"/_api/version\", \"\", %{\"header\" => \"new_value\"})\niex> request.headers\n%{\"header\" => \"new_value\"}\n```","ref":"readme.html#headers"},{"type":"extras","title":"Transport - Arangox","doc":"The `:connect_timeout` start option defaults to `5_000`.\n\nTransport options can be specified via `:tcp_opts` and `:ssl_opts`, for unencrypted and\nencrypted connections respectively. When using `:gun` or `:mint`, these options are passed\ndirectly to the `:transport_opts` connect option.\n\nSee [`:gen_tcp.connect_option()`](http://erlang.org/doc/man/gen_tcp.html#type-connect_option)\nfor more information on `:tcp_opts`,\nor [`:ssl.tls_client_option()`](http://erlang.org/doc/man/ssl.html#type-tls_client_option) for `:ssl_opts`.\n\nThe `:client_opts` option can be used to pass client-specific options to `:gun` or `:mint`.\nThese options are merged with and may override values set by arangox. Some options cannot be\noverridden (i.e. `:mint`'s `:mode` option). If `:transport_opts` is set here it will override\neverything given to `:tcp_opts` or `:ssl_opts`, regardless of whether or not a connection is\nencrypted.\n\nSee the `gun:opts()` type in the [gun docs](https://ninenines.eu/docs/en/gun/1.3/manual/gun/)\nor [`connect/4`](https://hexdocs.pm/mint/Mint.HTTP.html#connect/4) in the mint docs for more\ninformation.","ref":"readme.html#transport"},{"type":"extras","title":"Request Options - Arangox","doc":"Request options are handled by and passed directly to `:db_connection`.\nSee [execute/4](https://hexdocs.pm/db_connection/DBConnection.html#execute/4) in the `:db_connection` docs for supported\noptions.\n\nRequest timeouts default to `15_000`.\n\n```elixir\niex> {:ok, conn} = Arangox.start_link()\niex> %Arangox.Response{status: 200, body: %{\"code\" => 200, \"error\" => false, \"mode\" => \"default\"}} = Arangox.get!(conn, \"/_admin/server/availability\", [], timeout: 15_000)\n```","ref":"readme.html#request-options"},{"type":"extras","title":"Contributing - Arangox","doc":"```\nmix format\nmix do format, credo --strict\ndocker-compose up -d\nmix test\n```","ref":"readme.html#contributing"},{"type":"extras","title":"Roadmap - Arangox","doc":"- `:get_endpoints` and `:port_mappings` options\n- An Ecto adapter\n- More descriptive logs","ref":"readme.html#roadmap"}],"content_type":"text/markdown","producer":{"name":"ex_doc","version":[48,46,51,49,46,50]}}